import os
import wget
import stat
import requests
import warnings

def check_internet():
    url='http://www.google.com/'
    timeout=5
    try:
        _ = requests.get(url, timeout=timeout)
        return True
    except requests.ConnectionError:
        return False

# Download precalculated weight files used for the machine learning algorithm
# and available from the repository.
def download_weights( weights_path ):
	if check_internet():
		# We get the content of the weights folder.
		z = requests.get('https://api.github.com/repos/koopa31/SegSRGAN/contents/data/weights?ref=master')
		contents = z.json()
		# Creation of weights folder if it does not already exist.
		if os.path.isdir(weights_path) is False:
			print(weights_path)
			os.mkdir(weights_path, mode=0o777)
		# Downloading of the files.
		for content in contents:
			if os.path.isfile(os.path.join(weights_path, content['name'])) is False:
				print("Processing %s" % content['path'])
				wget.download(content['download_url'], out=weights_path)
	else :
		warnings.warn("The internet connection doesn't work so the local weights set cannot be updated with the remote one. May cause issue of non existing file")
	


import numpy as np


class Normalization():
    def __init__(self, LR, hr):
        self.hr = hr
        self.LR = LR

    def get_normalized_image(self):
        max_value = np.max(self.LR)
        normalized_reference_image = self.hr / max_value
        normalized_low_resolution_image = self.LR / max_value
        return normalized_low_resolution_image, normalized_reference_image


import numpy as np
import scipy
import SimpleITK as sitk

class Interpolation():
    def __init__(self, normalized_low_resolution_image, upscale, order, interp):
        self.upscale = upscale
        self.order = order
        self.interp = interp
        self.normalized_low_resolution_image = normalized_low_resolution_image

    def get_interpolated_image(self, original_LR=None):
        if self.interp == 'scipy':
            interpolated_image, self.upscale = self.get_scipy_interpolation()
        elif self.interp == 'sitk':
            interpolated_image, self.upscale = self.get_sitk_interpolation(original_LR)
        return interpolated_image, self.upscale

    def get_scipy_interpolation(self):
        normalized_low_resolution_image = self.normalized_low_resolution_image
        interpolated_image = scipy.ndimage.zoom(normalized_low_resolution_image, zoom=self.upscale, order=self.order)
        return interpolated_image, self.upscale

    def get_sitk_interpolation(self, original_LR):
        normalized_low_resolution_image = sitk.GetImageFromArray(np.swapaxes(self.normalized_low_resolution_image, 0, 2))
        if type(original_LR) == type(normalized_low_resolution_image):
            normalized_low_resolution_image.SetSpacing(original_LR.GetSpacing())
            normalized_low_resolution_image.SetOrigin(original_LR.GetOrigin())
            normalized_low_resolution_image.SetDirection(original_LR.GetDirection())
        else:
            normalized_low_resolution_image.SetSpacing(original_LR.itk_image.GetSpacing())
            normalized_low_resolution_image.SetOrigin(original_LR.itk_image.GetOrigin())
            normalized_low_resolution_image.SetDirection(original_LR.itk_image.GetDirection())
        self.upscale = [int(round(i)) for i in self.upscale]
        interp = sitk.Expand(normalized_low_resolution_image, list(self.upscale), sitk.sitkBSpline)
        interp = np.swapaxes(sitk.GetArrayFromImage(interp), 0, 2)
        return interp, self.upscale


def swapList(newList):
    size = len(newList)

    # Swapping
    temp = newList[0]
    newList[0] = newList[size - 1]
    newList[size - 1] = temp

    return newList